/*
 * AbstractDomainObject.java
 *
 * 
 */

package com.iwombat.foundation;

import java.io.Serializable;
/**
 *
 * @author  brentp
 */
public abstract class AbstractDomainObject implements DomainObject, Serializable {

    private boolean isDirty;
    private boolean isNew;
    private Identifier id;

    /**
     * creates an instance with a new Identifier
     */
    
    public AbstractDomainObject(){
        setId(IdentifierFactory.createUniqueIdentifier());
    }

    /**
     * creates an instance with the specified Identifier
     */
    public AbstractDomainObject(Identifier id){
        setId(id);
    }
    
    /**
     * retrieves the current id of this Domain Object
     *
     * @return the id representing this Domain Object
     */
    public Identifier getId() {
        return this.id;
    }
    
    /**
     * set the current id for this domain object
     *
     *
     * @param id the new id to set for this Domain Object
     *
     */
    public void setId(Identifier id){
            this.id=id;
    }
    
    public boolean getIsDirty(){
    
        return this.isDirty;
    }
    
    
    public void setIsDirty(boolean isDirty) {
        this.isDirty=isDirty;
    }
    
    public void setIsNew(boolean newIsNew){
        this.isNew=newIsNew;
    }
    
    public boolean getIsNew(){
        return this.isNew;
    }

    /**
     * determines if another DomainObject is equal.
     * Delegates equality to the Identifier
     *
     * @param obj - the object to check equality
     */
    public boolean equals(Object obj) {
	try {
	   DomainObject domainObj = (DomainObject)obj;
           return this.getId().equals(domainObj.getId());
	} catch (ClassCastException e) {
		return false;
	}
    }

    /**
     * generates a hash code for the domain object
     * delegates hash code generation to the identifier
     */
    public int hashCode() {
	return this.getId().hashCode();
    }
    
}
