// (c) Copyright 2001-2002 iwombat.com all rights reserved
package com.iwombat.foundation.persist;

import java.sql.*;

import java.lang.reflect.*;

/**
 * Maps result set values to bean properties
 *
 * @author  $Author: brentp $
 * @version $Revision: 1.1 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/11/18 16:04:40 $</dd></dl>
 */

public interface ResultSetToBeanMapper {

    /**
     * sets a Bean property in the currently set target Bean with a value from the currently set result set
     * based on the specified query result to bean mapping configuration
     */
    public void setBeanProperty(QueryResultToBeanMapItem queryResultToBeanConfig);
    
    /**
     * sets the current target Bean
     * 
     * @param the target Bean to use
     */
    public void setTargetBean(Object targetBean);
    
    /**
     * retrieves the currently set target Bean
     *
     * @return the currently set target Bean
     */
    public Object getTargetBean();
    
    /**
     * sets the source result set
     *
     * @param the source result set to use
     */
    public void setSourceResultSet(ResultSet resultSet);
    
    /**
     * retrieves the currently set source result set
     *
     * @return the currently set source result set
     */
    public ResultSet getSourceResultSet();
    
    /**
     * sets an Identifier object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setIdentifierBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;

    /**
     * sets a String object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setStringBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;
    
    /**
     * sets a Long object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setLongBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;
    
    /**
     * sets a Float object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setFloatBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;
    
    /**
     * sets a Boolean object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setBooleanBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;
    
    /**
     * sets a java.util.Date object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setDateBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;
    
    /**
     * sets a ByteArray object to the target Bean
     *
     * @param the mutuator method to use on the target Bean
     * @param the column name to use on the source result set
     */
    public void setByteArrayBeanProperty(Method mutatorMethod, String columnName) throws java.sql.SQLException;
    
}
