// (c) 2003 Aegis Design - all rights reserved
package com.iwombat.swing;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * OpenList.java
 *
 * @author  $Author: bob $
 * @version $Revision: 1.2 $
 *
 * <br>
 * <dl><dt><b>Date::</b><dd>$Date: 2003/10/08 05:16:47 $</dd></dl>
 */

public class ListChooser extends JPanel 
  implements ListSelectionListener, ActionListener
{
  protected JLabel title;
  protected JTextField textField;
  protected JList list;
  protected JScrollPane scrollPane;
  
  private static final int ROWCOUNT = 4;
  
  public ListChooser(Object[] data, String listTitle) {
    setLayout(null);
    title = new JLabel(listTitle, JLabel.LEFT);
    add(title);
    textField = new JTextField();
    textField.addActionListener(this);
    add(textField);
    list = new JList(data);
    list.setVisibleRowCount(ROWCOUNT);
    list.addListSelectionListener(this);
    scrollPane = new JScrollPane(list);
    add(scrollPane);
  }

  public JList getList() {
      return list;
  }
  
  public void setSelected(String sel) {
    list.setSelectedValue(sel, true);
    textField.setText(sel);
  }

  public String getSelected() { return textField.getText(); }

  public void setSelectedInt(int value) {
    setSelected(Integer.toString(value));
  }

  public int getSelectedInt() {
    try { 
      return Integer.parseInt(getSelected());
    }
    catch (NumberFormatException ex) { return -1; }
  }

  public void valueChanged(ListSelectionEvent e) {
    Object obj = list.getSelectedValue();
    if (obj != null)
      textField.setText(obj.toString());
  }

  public void actionPerformed(ActionEvent e) {
    ListModel model = list.getModel();
    String key = textField.getText().toLowerCase();
    for (int k=0; k<model.getSize(); k++) {
      String data = (String)model.getElementAt(k);
      if (data.toLowerCase().startsWith(key)) {
        list.setSelectedValue(data, true);
        break;
      }
    }
  }

  public void addListSelectionListener(ListSelectionListener listener) {
    list.addListSelectionListener(listener);
  }

  public Dimension getPreferredSize() {
    Insets ins = getInsets();
    Dimension d1 = title.getPreferredSize();
    Dimension d2 = textField.getPreferredSize();
    Dimension d3 = scrollPane.getPreferredSize();
    int w = Math.max(Math.max(d1.width, d2.width), d3.width);
    int h = d1.height + d2.height + d3.height;
    return new Dimension(w+ins.left+ins.right, 
      h+ins.top+ins.bottom);
  }

  public Dimension getMaximumSize() {
    Insets ins = getInsets();
    Dimension d1 = title.getMaximumSize();
    Dimension d2 = textField.getMaximumSize();
    Dimension d3 = scrollPane.getMaximumSize();
    int w = Math.max(Math.max(d1.width, d2.width), d3.width);
    int h = d1.height + d2.height + d3.height;
    return new Dimension(w+ins.left+ins.right, 
      h+ins.top+ins.bottom);
  }

  public Dimension getMinimumSize() {
    Insets ins = getInsets();
    Dimension d1 = title.getMinimumSize();
    Dimension d2 = textField.getMinimumSize();
    Dimension d3 = scrollPane.getMinimumSize();
    int w = Math.max(Math.max(d1.width, d2.width), d3.width);
    int h = d1.height + d2.height + d3.height;
    return new Dimension(w+ins.left+ins.right, 
      h+ins.top+ins.bottom);
  }

  public void doLayout() {
    Insets ins = getInsets();
    Dimension d = getSize();
    int x = ins.left;
    int y = ins.top;
    int w = d.width-ins.left-ins.right;
    int h = d.height-ins.top-ins.bottom;

    Dimension d1 = title.getPreferredSize();
    title.setBounds(x, y, w, d1.height);
    y += d1.height;
    Dimension d2 = textField.getPreferredSize();
    textField.setBounds(x, y, w, d2.height);
    y += d2.height;
    scrollPane.setBounds(x, y, w, h-y);
  }
}
