// (c) Copyright 2001-2002 iwombat.com all rights reserved

package com.iwombat.util;

import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

/**
 * Variant implements a weakly-typed container for strongly typed primitives and base types.
 * The primary use of variants is with JDBC and column bindings. Accessor and mutator
 * methods are vast.
 *
 * @author  $Author: bob $
 * @version $Revision: 1.6 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/10/01 18:08:01 $</dd></dl>
 */


public class Variant implements Cloneable, Serializable {

    private VariantEnum        type         = null;
    private int                intVal       = 0;
    private long               longVal      = 0l;
    private float              floatVal     = 0.0f;
    private double             doubleVal    = 0.0;
    private boolean            booleanVal   = false;
    private byte[]             byteArrayVal = null;
    private String             stringVal    = "";
    private java.sql.Timestamp timestampVal = null;
    private transient Object   objectVal    = null;

    /**
     * Constructor Variant
     * Description: Create a Variant of no type and value
     */

    public Variant() {}

    /**
     * Constructor Variant
     * Description: Create a Variant of a specific type. If this constructor is used,
     * all set operations must be of dataType that the Variant was constructed
     * with.
     *
     * @param The Datatype of the variant
     * @see VariantEnum
     */

    public Variant(VariantEnum thetype) {
        type = thetype;
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.BOOLEAN, with an initial value.
     *
     * @param initial value
     * @see VariantEnum
     */

    public Variant(boolean value) {
        type       = VariantEnum.BOOLEAN;
        this.setboolean(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.CHAR, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(char value) {
        type   = VariantEnum.CHAR;
        this.setchar(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.BYTE, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(byte value) {
        type   = VariantEnum.BYTE;
        this.setbyte(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.SHORT, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(short value) {
        type   = VariantEnum.SHORT;
        this.setshort(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.INT, with an initial value.
     *
     * @param value
     */

    public Variant(int value) {
        type   = VariantEnum.INT;
        this.setint(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.LONG, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(long value) {
        type    = VariantEnum.LONG;
        this.setlong(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.FLOAT, with an initial value.
     *
     * @param value
     */

    public Variant(float value) {
        type     = VariantEnum.FLOAT;
        this.setfloat(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.DOUBLE, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(double value) {
        type      = VariantEnum.DOUBLE;
        this.setdouble(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.BYTE_ARRAY, with an initial value.
     *
     * @param value
     */

    public Variant(byte[] value) {
        type         = VariantEnum.BYTE_ARRAY;
        this.setByteArray(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.STRING, with an initial value.
     *
     * @param value
     */

    public Variant(String value) {
        type      = VariantEnum.STRING;
        this.setString(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.SQL_TIMESTAMP, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(java.sql.Timestamp value) {
        type         = VariantEnum.SQL_TIMESTAMP;
        this.setJavaSqlTimestamp(value);
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.INT, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(Integer value) {

        type = VariantEnum.INT;

        if (value != null) {
            this.setint(value.intValue());
        }
    }

    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.LONG, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(Long value) {

        type = VariantEnum.LONG;

        if (value != null) {
            this.setlong(value.longValue());
        }
    }
    
    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.SHORT, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(Short value) {

        type = VariantEnum.SHORT;

        if (value != null) {
            this.setshort(value.shortValue());
        }
    }
    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.DOUBLE, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(Double value) {

        type = VariantEnum.DOUBLE;

        if (value != null) {
            this.setdouble(value.doubleValue());
        }
    }

    public Variant(Float value) {

        type = VariantEnum.FLOAT;

        if (value != null) {
            this.setfloat(value.floatValue());
        }
    }
    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.BYTE, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(Byte value) {

        type = VariantEnum.BYTE;

        if (value != null) {
            this.setbyte(value.byteValue());
        }
    }

    public Variant(Character value) {

        type = VariantEnum.CHAR;

        if (value != null) {
            this.setchar(value.charValue());
        }
    }
    
    public Variant(Boolean value) {
        type       = VariantEnum.BOOLEAN;
        if (value != null) {
            this.setboolean(value.booleanValue());
        }     
    }
    /**
     * Constructor Variant
     * Description: Create a Variant of type VariantEnum.OBJECT, with an initial value.
     *
     * @param value
     * @see VariantEnum
     */

    public Variant(Object value) {
        type      = VariantEnum.OBJECT;               
        setObject(value);        
    }

    /**
     * Method getVariantType
     * Description: Get the type of this variant
     *
     * @return VariantEnum
     * @see VariantEnum
     */

    public VariantEnum getVariantType() {

        if (type == null) {
            return VariantEnum.UNKNOWN;
        }

        return type;
    }

    /**
     * Method typeId
     * Description: Return the enumerated type id of this variant
     *
     * @return int
     * @see VariantEnum
     */

    public int typeId() {

        if (type == null) {
            return VariantEnum.UNKNOWN.getValue();
        }

        return type.getValue();
    }

    
    
    private void typeProblem(VariantEnum unexpectedType, VariantEnum expectedType) {
        throw new VariantIllegalTypeException(
            "Variant of type " + expectedType.toString() + 
            " does not support translations to type " + unexpectedType.toString());
    }

    /**
     * Method get
     * Description: get returns the value of this instance as an Object. 
     * For primitave types,  their value is first wrapped in a standard Java 
     * class with the corresponding data type. For arrays, an reference to the
     * array is returned. 
     * If no type is assigned to this Variant, null is returned.
     *
     * @return the Variant's value, null if this variant has no type
     */

    public final Object get() {

        if (type == null) {
            return null;
        }

        if (type == VariantEnum.BOOLEAN) {
            return new Boolean(booleanVal);
        }

        if (type == VariantEnum.BYTE) {
            return new Byte((byte) intVal);
        }

        if (type == VariantEnum.BYTE_ARRAY) {
            return byteArrayVal;
        }

        if (type == VariantEnum.CHAR) {
            return new Character((char) intVal);
        }

        if (type == VariantEnum.DOUBLE) {
            return new Double(doubleVal);
        }

        if (type == VariantEnum.FLOAT) {
            return new Float(floatVal);
        }

        if (type == VariantEnum.INT) {
            return new Integer(intVal);
        }

        if (type == VariantEnum.LONG) {
            return new Long(longVal);
        }

        if (type == VariantEnum.OBJECT) {
            return objectVal;
        }

        if (type == VariantEnum.SHORT) {
            return new Short((short) intVal);
        }

        if (type == VariantEnum.SQL_TIMESTAMP) {
            return timestampVal;
        }

        if (type == VariantEnum.STRING) {
            return stringVal;
        }

        // we've run out of types to check, return a null
        return null;    
    }

    /**
     * Method getboolean
     * Description: Return the value of the variant as a boolean
     * @return value
     */

    public final boolean getboolean() {

        if (type != VariantEnum.BOOLEAN) {
            typeProblem(type, VariantEnum.BOOLEAN);
        }

        return booleanVal;
    }

    /**
     * Method getchar
     * Description: Return the value of the variant as a char
     * @return value
     */

    public final char getchar() {

        if (type != VariantEnum.CHAR) {
            typeProblem(type, VariantEnum.CHAR);
        }

        return (char) intVal;
    }

    /**
     * Method getbyte
     * Description: Return the value of the variant as a byte
     * @return value
     */

    public final byte getbyte() {

        if (type != VariantEnum.BYTE) {
            typeProblem(type, VariantEnum.BYTE);
        }

        return (byte) intVal;
    }

    /**
     * Method getshort
     * Description: Return the value of the variant as a short
     * @return value
     */

    public final short getshort() {

        if (type != VariantEnum.SHORT) {
            typeProblem(type, VariantEnum.SHORT);
        }

        return (short) intVal;
    }

    /**
     * Method getint
     * Description: Return the value of the variant as an int
     * @return value
     */

    public final int getint() {

        if (type != VariantEnum.INT) {
            typeProblem(type, VariantEnum.INT);
        }

        return intVal;
    }

    /**
     * Method getlong
     * Description: Return the value of the variant as a long
     * @return value
     */

    public final long getlong() {

        if (type != VariantEnum.LONG) {
            typeProblem(type, VariantEnum.LONG);
        }

        return longVal;
    }

    /**
     * Method getfloat
     * Description: Return the value of the variant as a float
     * @return value
     */

    public final float getfloat() {

        if (type != VariantEnum.FLOAT) {
            typeProblem(type, VariantEnum.FLOAT);
        }

        return floatVal;
    }

    /**
     * Method getdouble
     ** Description: Return the value of the variant as a double
     * @return value
     */

    public final double getdouble() {

        if (type != VariantEnum.DOUBLE) {
            typeProblem(type, VariantEnum.DOUBLE);
        }

        return doubleVal;
    }

    /**
     * Method getByteArray
     * Description: Return the value of the variant as a byte-array
     * @return value
     */

    public final byte[] getByteArray() {

        if (type != VariantEnum.BYTE_ARRAY) {
            typeProblem(type, VariantEnum.BYTE_ARRAY);
        }

        return byteArrayVal;
    }

    /**
     * Method getString
     * Description: Return the value of the variant as a String
     * @return value
     */

    public final String getString() {

        if (type != VariantEnum.STRING) {
            typeProblem(type, VariantEnum.STRING);
        }

        return stringVal;
    }

    /**
     * Method getJavaSqlTimestamp
     * Description: Return the value of the variant as a java.sql.Timestamp
     * @return value
     */

    public final java.sql.Timestamp getJavaSqlTimestamp() {

        if (type != VariantEnum.SQL_TIMESTAMP) {
            typeProblem(type, VariantEnum.SQL_TIMESTAMP);
        }

        return timestampVal;
    }

    /**
     * Method getObject
     * Description: Return the value of the variant as an Object
     * @return value
     */

    public final Object getObject() {

        if (type != VariantEnum.OBJECT) {
            typeProblem(type, VariantEnum.OBJECT);
        }

        return objectVal;
    }

    /**
     * Method getBoolean
     * Description: Return the value of the variant as a Boolean class
     * @return value
     */

    public final Boolean getBoolean() {

        if (type != VariantEnum.BOOLEAN) {
            typeProblem(type, VariantEnum.BOOLEAN);
        }

        return new Boolean(booleanVal);
    }

    /**
     * Method getCharacter
     * Description: Return the value of the variant as a Character class
     * @return value
     */

    public final Character getCharacter() {

        if (type != VariantEnum.CHAR) {
            typeProblem(type, VariantEnum.CHAR);
        }

        return new Character((char) intVal);
    }

    /**
     * Method getByte
     * Description: Return the value of the variant as a Byte class
     * @return value
     */

    public final Byte getByte() {

        if (type != VariantEnum.BYTE) {
            typeProblem(type, VariantEnum.BYTE);
        }

        return new Byte((byte) intVal);
    }

    /**
     * Method getShort
     * Description: Return the value of the variant as a Short class
     * @return value
     */

    public final Short getShort() {

        if (type != VariantEnum.SHORT) {
            typeProblem(type, VariantEnum.SHORT);
        }

        return new Short((short) intVal);
    }

    /**
     * Method getInteger
     * Description: Return the value of the variant as an Integer class
     * @return value
     */

    public final Integer getInteger() {

        if (type != VariantEnum.INT) {
            typeProblem(type, VariantEnum.INT);
        }

        return new Integer(intVal);
    }

    /**
     * Method getLong
     * Description: Return the value of the variant as a Long class
     * @return value
     */

    public final Long getLong() {

        if (type != VariantEnum.LONG) {
            typeProblem(type, VariantEnum.LONG);
        }

        return new Long(longVal);
    }

    /**
     * Method getFloat
     * Description: Return the value of the variant as a Float class
     * @return value
     */

    public final Float getFloat() {

        if (type != VariantEnum.FLOAT) {
            typeProblem(type, VariantEnum.FLOAT);
        }

        return new Float(floatVal);
    }

    /**
     * Method getDouble
     * Description: Return the value of the variant as a Double class
     * @return value
     */

    public final Double getDouble() {

        if (type != VariantEnum.DOUBLE) {
            typeProblem(type, VariantEnum.DOUBLE);
        }

        return new Double(doubleVal);
    }


    // a bunch of proxies for the proper set methods
    public final void set(boolean val) { setboolean(val);  }
    public final void set(char val)    { setchar(val);     }
    public final void set(byte val)    { setbyte(val);     }
    public final void set(short val)   { setshort(val);    }
    public final void set(int val)     { setint(val);      }
    public final void set(long val)    { setlong(val);     }
    public final void set(float val)   { setfloat(val);    }
    public final void set(double val)  { setdouble(val);   }
    public final void set(byte[] val)  { setByteArray(val);}
    
    // we're not supporting null values for wrapper type objects
    public final void set(Boolean val)   { setboolean(val.booleanValue()); }
    public final void set(Character val) { setchar(val.charValue());       }
    public final void set(Byte val)      { setbyte(val.byteValue());       }
    public final void set(Short val)     { setshort(val.shortValue());     }
    public final void set(Integer val)   { setint(val.intValue());         }
    public final void set(Long val)      { setlong(val.longValue());       }
    public final void set(Float val)     { setfloat(val.floatValue());     }
    public final void set(Double val)    { setdouble(val.doubleValue());   }
    
    public final void set(String val)    { setString(val);                 }
    public final void set(Object val)    { setObject(val);                 }
   
    public final void set(java.sql.Timestamp val) { setJavaSqlTimestamp(val);}

    /**
     * Method setboolean
     * Description: Set the value of the variant to a Boolean class
     * @param value
     */

    public final void setboolean(boolean val) {

        if ((type != null) && (type != VariantEnum.BOOLEAN)) {
            typeProblem(type, VariantEnum.BOOLEAN);
        }

        type       = VariantEnum.BOOLEAN;
        booleanVal = val;
    }

    /**
     * Method setchar
     * Description: Set the value of the variant to a char
     * @param value
     */

    public final void setchar(char val) {

        if ((type != null) && (type != VariantEnum.CHAR)) {
            typeProblem(type, VariantEnum.CHAR);
        }

        type   = VariantEnum.CHAR;
        intVal = val;
    }

    /**
     * Method setbyte
     * Description: Set the value of the variant to a byte
     * @param value
     */

    public final void setbyte(byte val) {

        if ((type != null) && (type != VariantEnum.BYTE)) {
            typeProblem(type, VariantEnum.BYTE);
        }

        type   = VariantEnum.BYTE;
        intVal = val;
    }

    /**
     * Method setshort
     * Description: Set the value of the variant to a short
     * @param value
     */

    public final void setshort(short val) {

        if ((type != null) && (type != VariantEnum.SHORT)) {
            typeProblem(type, VariantEnum.SHORT);
        }

        type   = VariantEnum.SHORT;
        intVal = val;
    }

    /**
     * Method setint
     * Description: Set the value of the variant to an int
     * @param value
     */

    public final void setint(int val) {

        if ((type != null) && (type != VariantEnum.INT)) {
            typeProblem(type, VariantEnum.INT);
        }

        type   = VariantEnum.INT;
        intVal = val;
    }

    /**
     * Method setlong
     * Description: assigns the given long value to this instance.
     *
     * @param val long value to assign.
     */

    public final void setlong(long val) {

        if ((type != null) && (type != VariantEnum.LONG)) {
            typeProblem(type, VariantEnum.LONG);
        }

        type    = VariantEnum.LONG;
        longVal = val;
    }

    /**
     * Method setfloat
     * Description: Set the value of the variant to a float
     * @param value
     */

    public final void setfloat(float val) {

        if ((type != null) && (type != VariantEnum.FLOAT)) {
            typeProblem(type, VariantEnum.FLOAT);
        }

        type     = VariantEnum.FLOAT;
        floatVal = val;
    }

    /**
     * Method setdouble
     * Description: Set the value of the variant to a double
     * @param value
     */

    public final void setdouble(double val) {

        if ((type != null) && (type != VariantEnum.DOUBLE)) {
            typeProblem(type, VariantEnum.DOUBLE);
        }

        type      = VariantEnum.DOUBLE;
        doubleVal = val;
    }

    /**
     * Method setByteArray
     * Description: Set the value of the variant to a byte-array
     * @param value
     */

    public final void setByteArray(byte[] val) {

        if ((type != null) && (type != VariantEnum.BYTE_ARRAY)) {
            typeProblem(type, VariantEnum.BYTE_ARRAY);
        }

        type         = VariantEnum.BYTE_ARRAY;
        byteArrayVal = val;
    }

    /**
     * Method setString
     * Description: Set the value of the variant to a String
     * @param value
     */

    public final void setString(String val) {

        if ((type != null) && (type != VariantEnum.STRING)) {
            typeProblem(type, VariantEnum.STRING);
        }

        type      = VariantEnum.STRING;
        if (null == val) {
            stringVal = null;
        } else {
            stringVal = new String(val);
        }
    }

    /**
     * Method setJavaSqlTimestamp
     * Description: Set the value of the variant to a java.sql.Timestamp
     * @param value
     */

    public final void setJavaSqlTimestamp(java.sql.Timestamp val) {

        if ((type != null) && (type != VariantEnum.SQL_TIMESTAMP)) {
            typeProblem(type, VariantEnum.SQL_TIMESTAMP);
        }

        type         = VariantEnum.SQL_TIMESTAMP;
        if (null == val) {
            timestampVal = null;
        } else {
            timestampVal = new java.sql.Timestamp(val.getTime());
        }
    }

    /**
     * Method setObject
     * Description: Set the value of the variant to an Object
     * @param value
     */

    public final void setObject(Object val) {
        
        if ((type != null) && (type != VariantEnum.OBJECT)) {
            typeProblem(type, VariantEnum.OBJECT);
        }
        
        type      = VariantEnum.OBJECT;
        objectVal = val;
    }

    /**
     * Method toString
     * Description: get the value of the Variant as a String
     * always returns a valid string.
     * @return String representation of the current value if value is null
     *      we return an empty string
     */

    public String toString() {

        if (type == null) {
            return VariantEnum.UNKNOWN.toString();
        }

        String rString = "";

        if (type == VariantEnum.BOOLEAN) {
            rString = (booleanVal) ? "true" : "false";
        } else if (type == VariantEnum.BYTE) {
            rString = Byte.toString((byte) intVal);
        } else if (type == VariantEnum.BYTE_ARRAY) {
            rString = ((byteArrayVal == null) ? "" : new String(byteArrayVal));
        } else if (type == VariantEnum.CHAR) {
            char[] chars = new char[]{ (char) intVal };
            rString = new String(chars);
        } else if (type == VariantEnum.DOUBLE) {
            rString = Double.toString(doubleVal);
        } else if (type == VariantEnum.FLOAT) {
            rString = Float.toString(floatVal);
        } else if (type == VariantEnum.INT) {
            rString = Integer.toString(intVal);
        } else if (type == VariantEnum.LONG) {
            rString = Long.toString(longVal);
        } else if (type == VariantEnum.OBJECT) {
            rString = (objectVal == null) ? "" : objectVal.toString();
        } else if (type == VariantEnum.SHORT) {
            rString = Short.toString((short) intVal);
        } else if (type == VariantEnum.SQL_TIMESTAMP) {
            rString = (timestampVal == null) ? "" : timestampVal.toString();
        } else if (type == VariantEnum.STRING) {
            rString = (stringVal == null) ? "" : stringVal;
        }

        return rString;
    }

    /**
     * Method clone
     * Description: Make a copy of this instance
     * @return Object
     */

    public Object clone() {

        Variant value = new Variant(type);

        value.type       = type;
        value.intVal     = intVal;
        value.longVal    = longVal;
        value.floatVal   = floatVal;
        value.doubleVal  = doubleVal;
        value.booleanVal = booleanVal;

        if (byteArrayVal != null) {
            value.byteArrayVal = new byte[byteArrayVal.length];

            System.arraycopy(byteArrayVal, 0, value.byteArrayVal, 0,
                             byteArrayVal.length);
        } else {
            value.byteArrayVal = null;
        }

        if (stringVal != null) {
            value.stringVal = new String(stringVal);
        } else {
            value.stringVal = null;
        }

        if (timestampVal != null) {
            value.timestampVal = new java.sql.Timestamp(timestampVal.getTime());
        } else {
            value.timestampVal = null;
        }

        value.objectVal = objectVal;    // need some type of clone support!

        return value;
    }

    // serialization support
    private void writeObject(ObjectOutputStream s) throws IOException {

        s.defaultWriteObject();

        if (objectVal instanceof Serializable) {
            s.writeObject(objectVal);
        } else {
            s.writeObject(null);
        }
    }

    // serialization support
    private void readObject(ObjectInputStream s)
            throws IOException, ClassNotFoundException {

        s.defaultReadObject();

        objectVal = s.readObject();
    }
}
