// (c) 2001-2002 iWombat.com All rights reserved
package com.iwombat.foundation;

import junit.framework.*;

/**
 * Some simple tests for ObjectIds.
 *
 * @author  $Author: bob $
 * @version $Revision: 1.3 $
 * @date    $Date: 2002/02/10 20:18:35 $
 */
public class ObjectIdTest extends TestCase {
        static final int listLength = 200;
        static final ObjectId[] oidList = new ObjectId[listLength];
    
	public ObjectIdTest(String name) {
		super(name);
	}
        
	public void setUp() {
            for (int i=0; i<listLength; i++) {
               oidList[i] = new ObjectId();
            }
	}
        
	public static Test suite() {
		return new TestSuite(ObjectIdTest.class);
	}
        
        /**
         * Test out the equals method of objectid
         */
	public void testEquals() {
		ObjectId idOne = new ObjectId();
		ObjectId idTwo = new ObjectId();
		assertTrue("ObjectId is not equal to itself", idOne.equals(idOne));
		assertTrue("Two ObjectIds are equal", !idOne.equals(idTwo));
                
	}

        /** 
         * Test exporting an id as a hex string and recreating
         */
	public void testStringRecreate() {
		ObjectId idOne = new ObjectId();
		String idString = idOne.toHexString();
		ObjectId idOneAgain = new ObjectId(idString);
		assertTrue("Remanufactued ObjectId is not equal to itself",
			idOne.equals(idOneAgain));
	}
        
        /**
         * Create a pile of ObjectIds and make sure they are all unique
         * Check hash code requirements in same pass.
         */
        public void testUniqueAndHash() {
            
            // check that equal ObjectIds have equal hashes
            ObjectId thisIdAgain = new ObjectId(oidList[0].toHexString());
            assertTrue("Equal ObjectIds do not have equal hashes", 
                 (oidList[0].hashCode() == thisIdAgain.hashCode()));
                      
            for (int i=0; i<listLength; i++) {   
                for (int j=i+1; j<listLength; j++) {
                    assertTrue("Different ObjectIds equal", 
                                !oidList[i].equals(oidList[j]));
                    
                    // not an actual requirement for hashCodes but something to strive for
                    assertTrue("Different ObjectIds have same hash", 
                                !(oidList[i].hashCode() == oidList[j].hashCode()) );
                }
            }
        }
        
        
        
            
}
