package com.iwombat.util;

import junit.framework.*;

/**
 *
 * @author combs
 */
public class ByteArrayTest extends TestCase {
   private static final byte[] byteArray1 = {(byte)0xaa, (byte)0xbb, (byte)0xcc, (byte)0xdd, (byte)0xee, (byte)0xff};
   private static final byte[] byteArray2 = {(byte)0x11, (byte)0x22, (byte)0x33, (byte)0x44, (byte)0x55, (byte)0x66, (byte)0x77, (byte)0x88};
   private static final byte[] byteArray3 = {(byte)0x55, (byte)0x66, (byte)0x77, (byte)0x88};
   private static final byte[] byteArray4 = {(byte)0x11, (byte)0x22, (byte)0x33, (byte)0x44};
   private static final long longValue = 0x1122334455667788L;
   private static final long longValue2 = 0x55667788L;
   private static final int  intValue  = 0x11223344;
   private static final short shortValue = 0x1122;
        
    public ByteArrayTest(java.lang.String testName) {
        super(testName);
    }
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(ByteArrayTest.class);
        return suite;
    }
    
    // Add test methods here, they have to start with 'test' name.
    // for example:
    // public void testHello() {}
    public void testStaticConverters() {
        
        
        byte[] raw = ByteArray.toByteArray(longValue);
        long testLong = ByteArray.bytesToLong(raw);
        this.assertEquals("Long did not survive round trip",longValue, testLong);
        
        raw = ByteArray.toByteArray(intValue);
        int testInt = (int)ByteArray.bytesToLong(raw);
        this.assertEquals("Integer did not survive round trip",intValue, testInt);
        
        raw = ByteArray.toByteArray(shortValue);
        short testShort = (short)ByteArray.bytesToLong(raw);
        this.assertEquals("Short did not survive round trip",shortValue, testShort);
        
        assertTrue("Same value test failed", ByteArray.sameValue(byteArray1, byteArray1));
        
        raw = ByteArray.clipMSBArray(byteArray2, 4);
        assertTrue("MSB clip failed", ByteArray.sameValue(byteArray3, raw));
        
        raw = ByteArray.chompArray(byteArray2, 4);
        assertTrue("Chomp failed", ByteArray.sameValue(byteArray4, raw));
        
        /* TODO complete these tests when needs arrise
         *
         * float value test
         *
         * double value test
         */
        
    }
    
    public void testInstanceMethods() {
        ByteArray testArray1 = new ByteArray(byteArray1);
        ByteArray testArray2 = new ByteArray(byteArray1);
        
        assertTrue("Equality test failed", testArray1.equals(testArray2));
        
        assertEquals("Length reporting not correct", byteArray1.length, testArray1.getLength());
        
        assertTrue("Not returning correct array", 
                 ByteArray.sameValue(byteArray1, testArray1.byteArrayValue()));
    }
        
    
}
